@@ -1,4 +1,29 @@
 # $Log: CrossTable.R,v $
+# Revision 1.4  2004/05/25 02:57:01  warnes
+# Updates from Mark Schwartz.
+#
+# Revision 1.4  2004/03/01 schwartz
+#
+# - Thanks to Gabor Grothendieck for providing ideas and
+#   comments on the following updates
+# - Modified function to allow a single vector to be
+#   tabulated in 'x'
+# - Added 'chisq' argument to disable the generation and
+#   printing of the chi-square tests, producing a simple
+#   table with no tests. Default value = FALSE.
+# - Removed restriction on minimum 2 x 2 tables (ie. can do a
+#   1 x n table, but no stats will be generated
+# - Added code to create generic row/col names if argument 'x'
+#   is a matrix without dimnames
+# - Added  'max.width' argument to enable wrapped output for
+#   1 x n tables. If n > max.width, output will wrap to next
+#   line
+# - Cleaned up list return code
+# - Separated functions to print m x n table and 1 x n vector
+#   to enable flexibility in vector output formats and to faciliate
+#   future creation and separation of table generation and print methods.
+
+
 # Revision 1.3  2003/01/30 21:41:35  warnes
 #
 # - Removed argument 'correct' and now print separate corrected values
@@ -26,6 +51,9 @@
 #
 #
 # $Log: CrossTable.R,v $
+# Revision 1.4  2004/05/25 02:57:01  warnes
+# Updates from Mark Schwartz.
+#
 # Revision 1.3  2003/01/30 21:41:35  warnes
 #
 # - Removed argument 'correct' and now print separate corrected values
@@ -49,26 +77,52 @@
 
 CrossTable <- function (x, y,
                         digits = 3,
+                        max.width = 5,
                         expected = FALSE,
                         prop.r = TRUE,
                         prop.c = TRUE,
                         prop.t = TRUE,
+                        chisq = FALSE,
                         fisher = FALSE,
                         mcnemar = FALSE)
 {
-  # require(ctest)  # included in base and loaded automatically
 
+  # Ensure that max.width >= 1
+  if (max.width < 1)
+    stop("max.width must be >= 1")
+  # Set 'x' vector flag
+  vector.x <- FALSE
+  # Ensure that if (expected), a chisq is done
+  if (expected)
+    chisq <- TRUE
+  
   if (missing(y))
   {
-    # if only x is specified, it must be a 2 dimensional matrix
-    if (length(dim(x)) != 2)
-      stop("x must be a 2 dimensional matrix if y is not given")
-    if(any(dim(x) < 2))
-      stop("x must have at least 2 rows and columns")
-    if(any(x < 0) || any(is.na(x)))
-      stop("all entries of x must be nonnegative and finite")
-    else
+    # is x a vector?
+    if (is.null(dim(x)))
+    {
+      # Remove any unused factor levels
+      x <- factor(x)
+      t <- t(as.matrix(table(x)))
+      vector.x <- TRUE
+    }  
+    # is x a matrix?
+    else if (length(dim(x) == 2))
+    {
+      if(any(x < 0) || any(is.na(x)))
+        stop("all entries of x must be nonnegative and finite")
+      
+      # Add generic dimnames if required
+      # check each dimname separately, in case user has defined one or the other
+      if (is.null(rownames(x)))
+        rownames(x) <- paste("[", 1:nrow(x), ",]", sep = "")
+      if (is.null(colnames(x)))
+        colnames(x) <- paste("[,", 1:ncol(x), "]", sep = "")
+
       t <- x
+    }
+    else
+      stop("x must be either a vector or a 2 dimensional matrix, if y is not given")
   }
   else
   {
@@ -83,13 +137,17 @@ CrossTable <- function (x, y,
     x <- factor(x)
     y <- factor(y)
 
-    if((nlevels(x) < 2) || (nlevels(y) < 2))
-      stop("x and y must have at least 2 levels")
-
     # Generate table
     t <- table(x, y)
   }
 
+  # if t is not at least a 2 x 2, do not do stats
+  # even if any set to TRUE. Do not do col/table props
+  if (any(dim(t) < 2))
+  {  
+    prop.c <- prop.r <- chisq <- expected <- fisher <- mcnemar <- FALSE
+  }  
+  
   # Generate cell proportion of row
   CPR <- prop.table(t, 1)
 
@@ -128,11 +186,106 @@ CrossTable <- function (x, y,
   RowTotal <- formatC(RowTotal, width = CWidth, format = "s")
 
   # Perform Chi-Square Tests
-  if (all(dim(t) == 2))
-    CSTc <- chisq.test(t, correct = TRUE)
+  # Needs to be before the table output, in case (expected = TRUE)
+  if (chisq)
+  {  
+    if (all(dim(t) == 2))
+      CSTc <- chisq.test(t, correct = TRUE)
 
-  CST <- chisq.test(t, correct = FALSE)
+    CST <- chisq.test(t, correct = FALSE)
+  }
 
+  # Default print function for n x m table
+  print.CrossTable.default <- function()
+  {
+    # Print Column headings
+    if (exists("RowData"))
+    {
+      cat(SpaceSep1, "|", ColData, "\n")
+      cat(formatC(RowData, width = RWidth, format = "s"), formatC(dimnames(t)[[2]], width = CWidth, format = "s"),
+          RowTotal, sep = " | ", collapse = "\n")
+    }
+    else
+      cat(SpaceSep1, formatC(dimnames(t)[[2]], width = CWidth, format = "s"), RowTotal,
+          sep = " | ", collapse = "\n")
+
+    cat(RowSep1, rep(RowSep, ncol(t) + 1), sep = "|", collapse = "\n")
+
+    # Print table cells
+    for (i in 1:nrow(t))
+    {
+      cat(FirstCol[i], formatC(c(t[i, ], RS[i]), width = CWidth), sep = " | ", collapse = "\n")
+
+      if (expected)
+        cat(SpaceSep1, formatC(CST$expected[i, ], digits = digits, format = "f", width = CWidth), SpaceSep2,
+            sep = " | ", collapse = "\n")
+
+      if (prop.r)
+        cat(SpaceSep1, formatC(c(CPR[i, ], RS[i] / GT), width = CWidth, digits = digits, format = "f"),
+            sep = " | ", collapse = "\n")
+        
+      if (prop.c)
+        cat(SpaceSep1, formatC(CPC[i, ], width = CWidth, digits = digits, format = "f"), SpaceSep2,
+            sep = " | ", collapse = "\n")
+      
+      if (prop.t)
+        cat(SpaceSep1, formatC(CPT[i, ], width = CWidth, digits = digits, format = "f"), SpaceSep2,
+            sep = " | ", collapse = "\n")
+
+      cat(RowSep1, rep(RowSep, ncol(t) + 1), sep = "|", collapse = "\n")
+    }
+
+    # Print Column Totals
+    cat(ColTotal, formatC(c(CS, GT), width = CWidth), sep = " | ", collapse = "\n")
+    
+    if (prop.c)
+      cat(SpaceSep1, formatC(CS / GT, width = CWidth, digits = digits, format = "f"), SpaceSep2,
+          sep = " | ", collapse = "\n")
+
+    cat(RowSep1, rep(RowSep, ncol(t) + 1), sep = "|", collapse = "\n")
+  }
+
+  # Print function for 1 x n vector
+  print.CrossTable.vector <- function()
+  {
+    if (length(t) > max.width)
+    {
+      # set breakpoints for output based upon max.width
+      final.row <- length(t) %% max.width
+      max <- length(t) - final.row
+      # Define breakpoint indices for each row
+      start <- seq(1, max, max.width)
+      end <- start + (max.width - 1)
+      # Add final.row if required
+      if (final.row > 0)
+      {
+         start <- c(start, end[length(end)] + 1)
+         end <- c(end, end[length(end)] + final.row)
+      }
+    }
+    else
+    {
+      # Each value printed horizontally in a single row
+      start <- 1
+      end <- length(t)
+    }
+
+    SpaceSep3 <- paste(SpaceSep2, " ", sep = "")
+    
+    for (i in 1:length(start))
+    {
+      # print column labels
+      cat(SpaceSep2, formatC(dimnames(t)[[2]][start[i]:end[i]], width = CWidth, format = "s"),
+          sep = " | ", collapse = "\n")
+
+      cat(SpaceSep3, rep(RowSep, (end[i] - start[i]) + 1), sep = "|", collapse = "\n")
+      cat(SpaceSep2, formatC(t[, start[i]:end[i]], width = CWidth), sep = " | ", collapse = "\n")
+      cat(SpaceSep2, formatC(CPT[, start[i]:end[i]], width = CWidth, digits = digits, format = "f"),
+          sep = " | ", collapse = "\n")
+      cat(SpaceSep3, rep(RowSep, (end[i] - start[i]) + 1), sep = "|", collapse = "\n")
+      cat("\n\n")
+    }
+  }
 
   # Print Cell Layout
 
@@ -154,56 +307,27 @@ CrossTable <- function (x, y,
   cat("Total Observations in Table: ", GT, "\n")
   cat(rep("\n", 2))
 
-  # Print Column headings
-
-  if (exists("RowData"))
-  {
-    cat(SpaceSep1, "|", ColData, "\n")
-    cat(formatC(RowData, width = RWidth, format = "s"), formatC(dimnames(t)[[2]], width = CWidth, format = "s"), RowTotal, sep = " | ", collapse = "\n")
-  }
+  if (!vector.x)
+    print.CrossTable.default()
   else
-    cat(SpaceSep1, formatC(dimnames(t)[[2]], width = CWidth, format = "s"), RowTotal, sep = " | ", collapse = "\n")
-
-  cat(RowSep1, rep(RowSep, ncol(t) + 1), sep = "|", collapse = "\n")
-
-  # Print table cells
-  for (i in 1:nrow(t))
-  {
-    cat(FirstCol[i], formatC(c(t[i, ], RS[i]), width = CWidth), sep = " | ", collapse = "\n")
-
-    if (expected)
-      cat(SpaceSep1, formatC(CST$expected[i, ], digits = digits, format = "f", width = CWidth), SpaceSep2, sep = " | ", collapse = "\n")
-
-    if (prop.r)
-      cat(SpaceSep1, formatC(c(CPR[i, ], RS[i] / GT), width = CWidth, digits = digits, format = "f"), sep = " | ", collapse = "\n")
-    if (prop.c)
-      cat(SpaceSep1, formatC(CPC[i, ], width = CWidth, digits = digits, format = "f"), SpaceSep2, sep = " | ", collapse = "\n")
-    if (prop.t)
-      cat(SpaceSep1, formatC(CPT[i, ], width = CWidth, digits = digits, format = "f"), SpaceSep2, sep = " | ", collapse = "\n")
-
-    cat(RowSep1, rep(RowSep, ncol(t) + 1), sep = "|", collapse = "\n")
-  }
-
-  # Print Column Totals
-  cat(ColTotal, formatC(c(CS, GT), width = CWidth), sep = " | ", collapse = "\n")
-  if (prop.c)
-    cat(SpaceSep1, formatC(CS / GT, width = CWidth, digits = digits, format = "f"), SpaceSep2, sep = " | ", collapse = "\n")
-  cat(RowSep1, rep(RowSep, ncol(t) + 1), sep = "|", collapse = "\n")
+    print.CrossTable.vector()
 
   # Print Statistics
-
-  cat(rep("\n", 2))
-  cat("Statistics for All Table Factors\n\n\n")
-
-  cat(CST$method,"\n")
-  cat("------------------------------------------------------------\n")
-  cat("Chi^2 = ", CST$statistic, "    d.f. = ", CST$parameter, "    p = ", CST$p.value, "\n\n")
-
-  if (all(dim(t) == 2))
+  if (chisq)
   {
-    cat(CSTc$method,"\n")
+    cat(rep("\n", 2))
+    cat("Statistics for All Table Factors\n\n\n")
+
+    cat(CST$method,"\n")
     cat("------------------------------------------------------------\n")
-    cat("Chi^2 = ", CSTc$statistic, "    d.f. = ", CSTc$parameter, "    p = ", CSTc$p.value, "\n")
+    cat("Chi^2 = ", CST$statistic, "    d.f. = ", CST$parameter, "    p = ", CST$p.value, "\n\n")
+
+    if (all(dim(t) == 2))
+    {
+      cat(CSTc$method,"\n")
+      cat("------------------------------------------------------------\n")
+      cat("Chi^2 = ", CSTc$statistic, "    d.f. = ", CSTc$parameter, "    p = ", CSTc$p.value, "\n")
+    }
   }
 
   # Perform McNemar tests
@@ -264,46 +388,36 @@ CrossTable <- function (x, y,
   }
 
   cat(rep("\n", 2))
-
+  
   # Create list of results for invisible()
 
-  if (all(dim(t) == 2))
+  CT <- list(t = t, prop.row = CPR, prop.col = CPC, prop.tbl = CPT)
+  
+  if (any(chisq, fisher, mcnemar))
   {
-    CT <- list(t, CPR, CPC, CPT, CST, CSTc)
-    Tnames <- c("t", "prop.col", "prop.row", "prop.tbl", "chisq", "chisq.corr")
-
-    if (fisher)
+    if (all(dim(t) == 2))
     {
-      CT <- c(CT, list(FTt, FTl, FTg))
-      Tnames <- c(Tnames, "fisher.ts", "fisher.lt", "fisher.gt")
-    }
+      if (chisq)
+        CT <- c(CT, list(chisq = CST, chisq.corr = CSTc))
 
-    if (mcnemar)
+      if (fisher)
+        CT <- c(CT, list(fisher.ts = FTt, fisher.tl = FTl, fisher.gt = FTg))
+
+      if (mcnemar)
+        CT <- c(CT, list(mcnemar = McN, mcnemar.corr = McNc))
+    }
+    else
     {
-      CT <- c(CT, list(McN, McNc))
-      Tnames <- c(Tnames, "mcnemar", "mcnemar.corr")
-    }
-  }
-  else
-  {
-    CT <- list(t, CPR, CPC, CPT, CST)
-    Tnames <- c("t", "prop.col", "prop.row", "prop.tbl", "chisq")
+      if (chisq)
+        CT <- c(CT, list(chisq = CST))
 
-    if (fisher)
-    {
-      CT <- c(CT, list(FTt))
-      Tnames <- c(Tnames, "fisher.ts")
-    }
+      if (fisher)
+        CT <- c(CT, list(fisher.ts = FTt))
 
-    if (mcnemar)
-    {
-      CT <- c(CT, list(McN))
-      Tnames <- c(Tnames, "mcnemar")
+      if (mcnemar)
+        CT <- c(CT, list(mcnemar = McN))
     }
-  }
-
-  # Set names for CT
-  names(CT) <- Tnames
+  }  
 
   # return list(CT)
   invisible(CT)
