@@ -1,6 +1,9 @@
-# $Id: ci.R,v 1.7 2002/09/23 13:59:30 warnes Exp $
+# $Id: ci.R,v 1.8 2003/08/07 03:49:54 warnes Exp $
 #
 # $Log: ci.R,v $
+# Revision 1.8  2003/08/07 03:49:54  warnes
+# - Fixed incorrect denominator in standard error for mean in ci.default.
+#
 # Revision 1.7  2002/09/23 13:59:30  warnes
 # - Modified all files to include CVS Id and Log tags.
 #
@@ -11,7 +14,7 @@ ci  <-  function(x, confidence=0.95,alpha=1-confidence,...)
 
 ci.default <- function(x, confidence=0.95,alpha=1-confidence,na.rm=FALSE,...) {
   est <- mean(x, na.rm=na.rm)
-  stderr <-  sd(x, na.rm=na.rm)/sqrt(nobs(x)-1);
+  stderr <-  sd(x, na.rm=na.rm)/sqrt(nobs(x));
   ci.low  <- est + qt(alpha/2, nobs(x)-1) * stderr
   ci.high <- est - qt(alpha/2, nobs(x)-1) * stderr
   retval  <- c(
