@@ -10,27 +10,27 @@ set.seed(1234)
   tx <- t(x)
 
   # SVD directly on matrix is SLOW:
-  system.time( val.x <- svd(x)$u )
+  val.x <- svd(x)$u
 
   # SVD on t(matrix) is FAST:
-  system.time( val.tx <- svd(tx)$v )
+  val.tx <- svd(tx)$v
 
   # and the results are equivalent:
   max( abs(val.x) - abs(val.tx) )
 
   # Time gap dissapears using fast.svd:
-  system.time( val.x <- fast.svd(x)$u )
-  system.time( val.tx <- fast.svd(tx)$v )
+  val.x <- fast.svd(x)$u
+  val.tx <- fast.svd(tx)$v
   max( abs(val.x) - abs(val.tx) )
 
 
   library(stats)
 
   # prcomp directly on matrix is SLOW:
-  system.time( pr.x <- prcomp(x) )
+  pr.x <- prcomp(x)
 
   # prcomp.fast is much faster
-  system.time( fast.pr.x <- fast.prcomp(x) )
+  fast.pr.x <- fast.prcomp(x)
 
   # and the results are equivalent
   max( pr.x$sdev - fast.pr.x$sdev )
@@ -39,3 +39,4 @@ set.seed(1234)
 
   # (except for the last and least significant component):
   max( abs(pr.x$rotation[,50]) - abs(fast.pr.x$rotation[,50]) )
+
